<?php
namespace Belgian_Tax\XML;

use DomDocument;
use DOMElement;

/*
 * Copyright (C) 2014 dany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
/**
 * @file
 * @brief Build the XML for intracom listing
 *
 */

/**
 * @class
 * @brief Build the XML for intracom listing
 *
 */
class Client_Intracom extends Declaration_XML
{





    protected $intracomm_id ; //!< $intracomm_id  DB : TVA_BELGE.INTRACOMM.I_ID

    function __construct($intracomm_id)
    {
        $this->intracomm_id = $intracomm_id;
        $this->domdoc = new DOMDocument('1.0', 'UTF-8');
        $this->ns = 'http://www.minfin.fgov.be/IntraConsignment';
        $this->domdoc->formatOutput = true;
    }
    
    function append_root()
    {
        $this->listing_xml = $this->domdoc->createElementNS($this->ns, 'ns2:IntraConsignment');

        $root = $this->domdoc->appendChild($this->listing_xml);
        $xmls = $root->setAttribute("xmlns", "http://www.minfin.fgov.be/InputCommon");

        $listing = $this->domdoc->appendChild($root);

        $nb = $this->domdoc->createAttribute('IntraListingsNbr');
        $nb->value = 1;
        $listing->appendChild($nb);

        $this->domdoc->appendChild($root);
    }



    /**
     *
     * @brief Add the listing for the declarant

     * 
     * @code
      * <ns2:ClientListing VATAmountSum="00.00" TurnOverSum="1000.72" ClientsNbr="2" SequenceNumber="1">
     * @endcode
     * 
     */
    function append_client_listing()
    {
        $declarant = new Declarant();
        /*
         * Fetch from intracom_client
         */
        $client = new List_Client_intracom($this->intracomm_id);
        $result = $client->compute_value();

        // variable
        $amount_sum = $result['AmountSum'];


        $seqnb = 1;

        $commentaire = "";

        $declaration = $this->domdoc->createElementNS($this->ns, "ns2:IntraListing");
        $ai = $declaration->setAttribute('AmountSum', sprintf('%.2f', $result['AmountSum']));
        $ai = $declaration->setAttribute('ClientsNbr', $result['ClientNB']);
        $ai = $declaration->setAttribute('SequenceNumber', $seqnb);

        $dom_declarant=$this->build_declarant($declarant);

        $declaration->appendChild($dom_declarant);

       $periode = $this->build_periode($result['Period']);


        $declaration->appendChild($periode);

        $client->append_listing($this->domdoc, $declaration);

        $this->listing_xml->appendChild($declaration);
        $this->domdoc->appendChild($this->listing_xml);

    }



    function build_xml() {
        $this->append_root();
        $representative = new Representative();
        if ( ! empty( $representative->name )) {
            $this->append_representative($representative);
        }
        $this->append_client_listing();
        return;

    }

}
