<?php
namespace Belgian_Tax\XML;
use IText;
/*
 *   This file is part of NOALYSS.
 *
 *   NOALYSS is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   NOALYSS is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with NOALYSS; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/**
 * @brief contains declarant
  @code
  $vat_number = "0000000097";
  $name = "Nom Declarant";
  $street = "Rue du declarant";
  $postcode = "9999";
  $city = "TESTCITY";
  $countrycode = "BE";
  $email = "dany@alch.be";
  $phone = "000000000";
  @endcode
 */
// Copyright Author Dany De Bontridder danydb@aevalys.eu
require_once 'transform_sql.class.php';
class Declarant
{

    /**
     * name
     */
    var $name;

    /**
     * street
     */
    var $street;

    /**
     * Postcode
     */
    var $postcode;

    /**
     * city
     */
    var $city;

    /**
     * country code (BE)
     */
    var $countrycode;

    /**
     * email
     */
    var $email;

    /**
     * phone
     */
    var $phone;

    /**
     * vatnumber
     */
    var $vatnumber;
    /**
     * readOnly
     */
    var $readOnly;
    //<! Declarant_SQL
    var $data;
    function __construct()
    {
        $this->readOnly=false;
        $this->data=new \Belgian_Tax\Declarant_SQL(1);
        $this->from_db();

    }

    /**
     * @brief Load object Declarant from DB
     * @return void
     * @throws \Exception
     */
    function fromPost()
    {
        global $g_parameter;
        $http=new \HttpInput();
        $this->name =$http->post("p_dec_name","string", $g_parameter->MY_NAME);
        $this->street =$http->post("p_dec_street", "string",$g_parameter->MY_STREET);
        $this->postcode =$http->post("p_dec_postcode", "string",$g_parameter->MY_CP);
        $this->city =$http->post("p_dec_city", "string",$g_parameter->MY_COMMUNE);
        $this->countrycode =$http->post("p_dec_countrycode", "string","BE");
        $this->email =$http->post("p_dec_email", "string",null);
        $this->phone =$http->post("p_dec_phone", "string",$g_parameter->MY_TEL);
        $this->vatnumber =$http->post("p_dec_vatnumber","string" ,$g_parameter->MY_TVA);
//        $this->year=$http->post('p_year',"number",null);
    }

    function input($error=0,$errmsg="")
    {

        $h_name = new IText('p_dec_name', $this->name);
        $h_name->readOnly=$this->readOnly;
        $h_name->require=true;

        $h_vatnumber = new IText('p_dec_vatnumber', str_ireplace(array(' ','BE'),'',$this->vatnumber??""));
        $h_vatnumber->readOnly=$this->readOnly;
        $h_vatnumber->require=true;
        $h_vatnumber->extra=' pattern="[0-9]+"';

        $h_street = new IText('p_dec_street', $this->street??"");
        $h_street->readOnly=$this->readOnly;
        $h_street->require=true;

        $h_postcode = new IText('p_dec_postcode', $this->postcode??"");
        $h_postcode->readOnly=$this->readOnly;
        $h_postcode->require=true;
        $h_postcode->extra='pattern="[0-9]+"';

        $h_city = new IText('p_dec_city', $this->city??"");
        $h_city->readOnly=$this->readOnly;
        $h_city->require=true;

        $h_countrycode = new IText('p_dec_countrycode', $this->countrycode??"");
        $h_countrycode->readOnly=$this->readOnly;
        $h_countrycode->require=true;
        $h_countrycode->placeholder="BE";

        $h_email = new IText('p_dec_email', $this->email??"");
        $h_email->readOnly=$this->readOnly;
        $h_email->require=true;

        $h_phone = new IText('p_dec_phone',str_ireplace(array(' ','+'),'',$this->phone??""));
        $h_phone->extra=' required pattern="[0-9]+"';
        $h_phone->placeholder="9999999";

        $h_phone->readOnly=$this->readOnly;
        global $g_tva_parameter;
        require_once  $g_tva_parameter['template'].'/xml_belgian_tax_declarant-input.php';
    }

    function save()
    {
        $this->verify();
        $this->data->d_name=$this->name;
        $this->data->d_street=$this->street;
        $this->data->d_postcode=$this->postcode;
        $this->data->d_city=$this->city;
        $this->data->d_countrycode=$this->countrycode;
        $this->data->d_email=$this->email;
        $this->data->d_phone=$this->phone;
        $this->data->d_vat_number=str_ireplace("be", "", $this->vatnumber);
//        $this->data->d_periode=$this->year;
        if ( $this->data->exist()>0 )
            $this->data->update();
        else
            $this->data->insert();
    }
    function from_db()
    {
        global $cn;


        $this->name = $this->data->d_name;
        $this->street = $this->data->d_street;
        $this->postcode = $this->data->d_postcode;
        $this->city = $this->data->d_city;
        $this->countrycode = $this->data->d_countrycode;
        $this->email = $this->data->d_email;
        $this->phone = $this->data->d_phone;
        $this->vatnumber = $this->data->d_vat_number;
    }

    function display()
    {
        $this->readOnly=true;
        $this->input();
    }
    function verify()
    {
        /* -- email must be valide */
        if (!preg_match( '/^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})$/',$this->email))
        {
            throw new \Exception(_('Email incorrect'),24);
        }
        if ($this->countrycode != 'BE') {
            throw new \Exception(_('Code Pays incorrect: doit être BE'),25);
        }
        
        if ( ! preg_match('/^[0-9]+/',$this->phone)) {
            throw new \Exception(_('Numéro de téléphone incorrect : uniquement des chiffres'),26);
        }
        if ( ! preg_match('/^[0-9]+/',$this->vatnumber)) {
            throw new \Exception(_('Numéro de TVA incorrect : uniquement des chiffres'),27);
        }
    }
}

?>
