<?php

namespace Belgian_Tax\XML;

use DOMElement;
use DOMDocument;

/*
 *   This file is part of noalyss.
 *
 *   noalyss is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   noalyss is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with NOALYSS; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// Copyright Author Dany De Bontridder danydb@aevalys.eu 2002-2024

/*!\file
 * \brief  concern the periodic declaration
 */

/*!
 *
 * \class  
 *
 * \brief  concern the periodic declaration
 */

class Declaration_Period extends Declaration_XML
{
    protected $ext_tva; //!< row for DB: TVA_BELGE.DECLARATION_AMOUNT
    protected $aGrid;

    function __construct($declaration_amount_id)
    {
        $this->ext_tva= new \Belgian_Tax\Ext_Tva(\Dossier::connect());
        $this->ext_tva->da_id=$declaration_amount_id;
        $this->ext_tva->load();
        $this->domdoc = new DOMDocument('1.0', 'UTF-8');
        $this->ns = 'http://www.minfin.fgov.be/VATConsignment';
        $this->domdoc->formatOutput = true;
        $this->aGrid =array(
            "d00"=>"0",
            "d01"=>"1",
            "d02"=>"2",
            "d03"=>"3",
            "d44"=>"44",
            "d45"=>"45",
            "d46"=>"46",
            "d47"=>"47",
            "d48"=>"48",
            "d49"=>"49",
            "d81"=>"81",
            "d82"=>"82",
            "d83"=>"83",
            "d84"=>"84",
            "d85"=>"85",
            "d86"=>"86",
            "d87"=>"87",
            "d88"=>"88",
            "d54"=>"54",
            "d55"=>"55",
            "d56"=>"56",
            "d57"=>"57",
            "d61"=>"61",
            "d63"=>"63",
            "dxx"=>"xx",
            "d59"=>"59",
            "d62"=>"62",
            "d64"=>"64",
            "dyy"=>"yy",
            "d71"=>"71",
            "d72"=>"72",
            "d91"=>"91"

        );

    }

    function append_root()
    {
        $this->listing_xml = $this->domdoc->createElementNS($this->ns, 'ns2:VATConsignment');

        $root = $this->domdoc->appendChild($this->listing_xml);
        $xmls = $root->setAttribute("xmlns", "http://www.minfin.fgov.be/InputCommon");

        $listing = $this->domdoc->appendChild($root);

        $nb = $this->domdoc->createAttribute('VATDeclarationsNbr');
        $nb->value = 1;
        $listing->appendChild($nb);

        $this->domdoc->appendChild($root);
    }

    /**
     * @brief Make the XML of all amounts
     * @param DOMElement $declaration
     * @return void
     * @throws \DOMException
     */
    function append_listing(DOMElement $declaration )
    {


        $nb_grid= count($this->aGrid);
        $data=$this->domdoc->createElement("ns2:Data");

        foreach ($this->aGrid as $sql_col=>$grid)
        {

            if ( $this->ext_tva->$sql_col == 0 ) continue;
            if ( in_array($grid,array('xx','yy')) ) continue;
            $dom_data = $this->domdoc->createElement( "ns2:Amount");
            $dom_data->setAttribute('GridNumber', $grid);
            $amount = $this->domdoc->createTextNode(sprintf("%.2f",$this->ext_tva->$sql_col));
            $dom_data->appendChild($amount);
            $data->appendChild($dom_data);
        }
        $declaration->appendChild($data);

    }

    /**
     * @brief append 2 lines :
     * @code
     * <ns2:ClientListingNihil>NO</ns2:ClientListingNihil>
     * <ns2:Ask Restitution="NO"/>
     * @endcode
     * @param DOMElement $declaration
     * @return void
     */
    function append_extraLine(DOMElement $declaration)
    {
        $value_client_listing=($this->ext_tva->get_parameter("client_listing")==1)?"YES":"NO";
        $clientListing=$this->domdoc->createElement("ns2:ClientListingNihil",$value_client_listing);
        $declaration->appendChild($clientListing);
        $value_ask_restitution=($this->ext_tva->get_parameter("ask_restitution")==1)?"YES":"NO";
        $restitution=$this->domdoc->createElement("ns2:Ask");
        $restitution->setAttribute("Restitution", $value_ask_restitution);
        $declaration->appendChild($restitution);
    }
    function append_data()
    {
        $declarant = new Declarant();

        $seqnb = 1;

        $declaration = $this->domdoc->createElementNS($this->ns, "ns2:VATDeclaration");
        $ai = $declaration->setAttribute('SequenceNumber', $seqnb);


        $dom_declarant=$this->build_declarant($declarant);

        $declaration->appendChild($dom_declarant);

        $periode = $this->build_periode(
                            array(
                                "periodicity"=>$this->ext_tva->periodicity,
                                "periode_dec"=>$this->ext_tva->periode_dec,
                                "exercice"=>$this->ext_tva->exercice)
                            );
        $declaration->appendChild($periode);

        $this->append_listing($declaration);
        $this->append_extraLine($declaration);
        $this->listing_xml->appendChild($declaration);
        $this->domdoc->appendChild($this->listing_xml);

    }

    function build_xml()
    {
        $this->append_root();
        $representative = new Representative();
        if (!empty($representative->name)) {
            $this->append_representative($representative);
        }
        $this->append_data();
        return;

    }
}