<?php

namespace Belgian_Tax\XML;

use DomDocument;
use DOMElement;

/*
 *   This file is part of noalyss.
 *
 *   noalyss is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   noalyss is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with NOALYSS; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// Copyright Author Dany De Bontridder danydb@aevalys.eu 2002-2024

/*!\file
 * \brief
 *Mother class for Listing client : intra and Belgian
 *
 */

/*!
 *
 * \class
 * \brief
 *Mother class for Listing client : intra and Belgian
 */

class Declaration_XML
{
    var $ns;//!< $ns namespace

    var $listing_xml; //!< all elements of the listing xml.
    /**
     *
     * @var DOMDocument
     */
    var $domdoc; //!< $domdoc Content the XML Document

    /*!
     * @brief : Build the DomElement Declarant
        * @code
        <ns2:Declarant>
               <VATNumber>0000000097</VATNumber>
               <Name>TEST NV-SA</Name>
               <Street>Av Test-laan 16</Street>
               <PostCode>9999</PostCode>
               <City>TESTCITY</City>
               <CountryCode>BE</CountryCode>
               <EmailAddress>cedric@francis.be</EmailAddress>
               <Phone>025555555</Phone>
           </ns2:Declarant>
        @endcode
        */
    function build_declarant( Declarant $p_declarant)
    {


        $declarant = $this->domdoc->createElementNS($this->ns, "ns2:Declarant");
        $declarant->appendChild($this->domdoc->createElement("VATNumber", $p_declarant->vatnumber??""));
        $declarant->appendChild($this->domdoc->createElement("Name", $p_declarant->name??""));
        $declarant->appendChild($this->domdoc->createElement("Street", $p_declarant->street??""));
        $declarant->appendChild($this->domdoc->createElement("PostCode", $p_declarant->postcode??""));
        $declarant->appendChild($this->domdoc->createElement("City", $p_declarant->city??""));
        $declarant->appendChild($this->domdoc->createElement("CountryCode", $p_declarant->countrycode??""));
        $declarant->appendChild($this->domdoc->createElement("EmailAddress", $p_declarant->email??""));
        $declarant->appendChild($this->domdoc->createElement("Phone", $p_declarant->phone??""));
        return $declarant;


    }

    /**
     * @brief Creation du mandataire non obligatoire
     * @code
     * <ns2:Representative>
     * <RepresentativeID identificationType="TIN" issuedBy="BE">0000000097</RepresentativeID>
     * <Name>Gevolmachtigde- Mandataire TEST NV-SA </Name>
     * <Street>AV Test-laan 8</Street>
     * <PostCode>9999</PostCode>
     * <City> TESTCITY</City>
     * <CountryCode>BE</CountryCode>
     * <EmailAddress>something@something.be</EmailAddress>
     * <Phone>02555555</Phone>
     * </ns2:Representative>
     * @endcode
     */
    function build_representative($p_representative)
    {

        $dom_representative = $this->domdoc->createElementNS($this->ns, "ns2:Representative");
        $rep_id = $this->domdoc->createElement("RepresentativeID");
        $at = $rep_id->setAttribute('identificationType', $p_representative->type);
        $ai = $rep_id->setAttribute('issuedBy', $p_representative->issued);
        $value = $this->domdoc->createTextNode($p_representative->id ?? "");
        $rep_id->appendChild($value);
        $dom_representative->appendChild($rep_id);
        $dom_representative->appendChild($this->domdoc->createElement("Name", $p_representative->name??""));
        $dom_representative->appendChild($this->domdoc->createElement("Street", $p_representative->street??""));
        $dom_representative->appendChild($this->domdoc->createElement("PostCode", $p_representative->postcode??""));
        $dom_representative->appendChild($this->domdoc->createElement("City", $p_representative->city??""));
        $dom_representative->appendChild($this->domdoc->createElement("CountryCode", $p_representative->countrycode??""));
        $dom_representative->appendChild($this->domdoc->createElement("EmailAddress", $p_representative->email??""));
        $dom_representative->appendChild($this->domdoc->createElement("Phone", $p_representative->phone??""));
        return $dom_representative;
    }
    function append_representative(Representative $p_representative)
    {

        $representative=$this->build_representative($p_representative);
        $this->listing_xml->appendChild($representative);
    }

    function toxml()
    {
        $str=$this->domdoc->saveXML();
        return $str;
    }

    /**
     * @brief build the domelement Periode
    *  @param $aPeriod array (periodicity : 1 Month/2 Quarter, periode_dec : month or quarter nb, exercice)
     * @return DOMElement
     * @throws \DOMException
     */
    function build_periode($aPeriod)
    {
        $periode = $this->domdoc->createElementNS($this->ns, "ns2:Period");

        if ( $aPeriod['periodicity']==1) {
            $periode->appendChild($this->domdoc->createElement("ns2:Month",$aPeriod['periode_dec']));
        } elseif ( $aPeriod['periodicity']==2)
        {
            $periode->appendChild($this->domdoc->createElement("ns2:Quarter",$aPeriod['periode_dec']));
        }
        $periode->appendChild($this->domdoc->createElement("ns2:Year",$aPeriod['exercice']));
        return $periode;
    }



}