<?php
namespace Belgian_Tax\XML;
use DOMElement;
use DOMDocument;
/*
 *   This file is part of NOALYSS.
 *
 *   NOALYSS is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   NOALYSS is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with NOALYSS; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// Copyright Author Dany De Bontridder danydb@aevalys.eu
/**
 * @brief manage the table intervat_client
 */

class List_Client
{

    protected $assujetti_id ; //!< $assujetti_id  DB : TVA_BELGE.ASSUJETTI.A_ID

    function __construct($assujetti_id)
    {
        $this->assujetti_id = $assujetti_id;
    }

    /**
     * @return mixed
     */
    public function getAssujettiId()
    {
        return $this->assujetti_id;
    }

    /**
     * @param mixed $assujetti_id
     */
    public function setAssujettiId($assujetti_id)
    {
        $this->assujetti_id = $assujetti_id;
        return $this;
    }

      /**
     * compute value from declarant
     * @returns array key = vat_amount_sum , turnoversum, Period(periodicity 1Month, 2Quarter,3Year
       *  ,periode_dec(month nb or quarter),exercice), ClientNB
     */
    function compute_value()
    {
        global $cn;
        $return = array()  ;
        $return["ClientNB"]   = $cn->get_value("select count(*) from tva_belge.assujetti_chld
                where 
                a_id=$1 "
                , array($this->assujetti_id));

        
        $return['vat_amount_sum'] = $cn->get_value("
            select sum(ac_vat::numeric) from 
            tva_belge.assujetti_chld where a_id=$1"
                , array($this->assujetti_id));
        
        $return['turnoversum'] = $cn->get_value("select sum(ac_amount::numeric) from
            tva_belge.assujetti_chld where a_id=$1"
              , array($this->assujetti_id));

        $return['Period']=$cn->get_row("select periodicity , periode_dec,exercice from tva_belge.assujetti  where a_id=$1",
            [$this->assujetti_id]);

        return $return;
    }

    /**
     * @brief make a listing : Add all the customers saved in declaration
     * @code
    <ns2:Client SequenceNumber="1">
    <ns2:CompanyVATNumber issuedBy="BE">0000000097</ns2:CompanyVATNumber>
    <ns2:TurnOver>500.36</ns2:TurnOver>
    <ns2:VATAmount>0.00</ns2:VATAmount>
    </ns2:Client>
    <ns2:Client SequenceNumber="2">
    <ns2:CompanyVATNumber issuedBy="BE">0000000097</ns2:CompanyVATNumber>
    <ns2:TurnOver>500.36</ns2:TurnOver>
    <ns2:VATAmount>0.00</ns2:VATAmount>
    </ns2:Client>
     * @endcode
     * @param DOMElement $p_dom
     * @param \DOMElement $declaration
     */

    function append_listing(DOMDocument $dom,DOMElement $declaration )
    {

        global $cn;

        $aClient=$cn->get_array("select * from tva_belge.assujetti_chld
         where a_id=$1"
            ,[$this->assujetti_id]);

        $nb_client=count($aClient);


        for ($i = 0; $i < $nb_client; $i++)
        {
            $client=$aClient[$i];
            $dom_client = $dom->createElement( "ns2:Client");
            $ai = $dom_client->setAttribute('SequenceNumber', $i+1);
            $company = $dom->createElement( "ns2:CompanyVATNumber");
            $company->setAttribute('issuedBy', 'BE');
            $de_vat_number = $dom->createTextNode(substr($client['ac_tvanum'],2));
            $company->appendChild($de_vat_number);
            $dom_client->appendChild($company);
            $dom_client->appendChild($dom->createElement("ns2:TurnOver"
                , sprintf("%.2f",$client['ac_amount'])));
            $dom_client->appendChild($dom->createElement("ns2:VATAmount"
                , sprintf("%.2f",$client['ac_vat'])));
            $declaration->appendChild($dom_client);
        }


    }

}

?>
