<?php
namespace Belgian_Tax\XML;
use IText;
use ISelect;
use INum;
/*
 *   This file is part of NOALYSS.
 *
 *   NOALYSS is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   NOALYSS is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with NOALYSS; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

// Copyright Author Dany De Bontridder danydb@aevalys.eu
/**
 * @brief contains the representative
  @code
  $ID = "0000000097";
  $issued = "BE";
  $type = 'TIN';
  $name = "Nom Mandataire";
  $street = "Nom de rue";
  $postcode = "9999";
  $city = "TESTCITY";
  $countrycode = "BE";
  $email = "dany@alch.be";
  $phone = "000000000";
  @endcode
 */

class Representative
{

    /**
     * id is the id of the listing
     */
    var $id;

    /**
     * issued by
     */
    var $issued;

    /**
     * Type (TIN)
     */
    var $type;

    /**
     * name
     */
    var $name;

    /**
     * street
     */
    var $street;

    /**
     * Postcode
     */
    var $postcode;

    /**
     * city
     */
    var $city;

    /**
     * country code (BE)
     */
    var $countrycode;

    /**
     * email
     */
    var $email;

    /**
     * phone
     */
    var $phone;

    /**
     * Transform_Representative_SQL
     */
    var $data;

    /**
     * readOnly
     */
    var $readOnly;

    function __construct()
    {
        $this->readOnly = false;
        $this->data = new \Belgian_Tax\Representative_SQL(1);
        $this->from_db();
    }

    function fromPost()
    {
        $http=new \HttpInput();
        $this->id = 1;
        $this->type = $http->post("p_type","string", null);
        $this->name = $http->post("p_name", "string",null);
        $this->street = $http->post("p_street","string", null);
        $this->postcode = $http->post("p_postcode", "string",null);
        $this->city = $http->post("p_city", "string",null);
        $this->countrycode = $http->post("p_countrycode", "string",null);
        $this->email = $http->post("p_email","string", null);
        $this->phone = $http->post("p_phone", "string",null);
        $this->issued = $http->post("p_issued","string", null);
    }

    function input($error=0,$errmsg="")
    {
        $h_type = new ISelect('p_type');
        $h_type->value = array(
            array("label" => 'TIN', "value" => "TIN"),
            array("label" => 'NVAT', "value" => "NVAT"),
            array("label" => 'other', "value" => "other")
        );
        $h_type->selected = $this->type;
        $h_type->readOnly = $this->readOnly;
        $h_name = new IText('p_name', $this->name);
        $h_name->readOnly = $this->readOnly;



        $h_street = new IText('p_street', $this->street);
        $h_street->readOnly = $this->readOnly;

        $h_postcode = new IText('p_postcode', $this->postcode);
        $h_postcode->readOnly = $this->readOnly;

        $h_city = new IText('p_city', $this->city);
        $h_city->readOnly = $this->readOnly;

        $h_countrycode = new IText('p_countrycode', $this->countrycode);
        $h_countrycode->readOnly = $this->readOnly;

        $h_email = new IText('p_email', $this->email);
        $h_email->readOnly = $this->readOnly;

        $h_phone = new IText('p_phone', str_ireplace(array(' ','+'),'',$this->phone??""));
        $h_phone->readOnly = $this->readOnly;

        $h_id = new INum('p_id', $this->id);
        $h_id->readOnly = $this->readOnly;

        $h_issued = new IText("p_issued", $this->issued);
        $h_issued->readOnly = $this->readOnly;
        global $g_tva_parameter;
        require_once  $g_tva_parameter['template'].'/xml_belgian_tax_representative-input.php';
    }

    function save()
    {
        $this->verify();
        $this->issued="BE";
        $this->data->rp_listing_id = $this->id;
        $this->data->rp_issued = $this->issued;
        $this->data->rp_type = $this->type;
        $this->data->rp_name = $this->name;
        $this->data->rp_street = $this->street;
        $this->data->rp_postcode = $this->postcode;
        $this->data->rp_countrycode = $this->countrycode;
        $this->data->rp_email = $this->email;
        $this->data->rp_phone = $this->phone;
        $this->data->rp_city=$this->city;
        if ($this->data->exist()>0)
            $this->data->update();
        else
            $this->data->insert();
    }

    function display()
    {
        $this->readOnly = true;
        $this->input();
    }

    function from_db()
    {
        global $cn;
        $this->issued = $this->data->rp_issued;
        $this->type = $this->data->rp_type;
        $this->name = $this->data->rp_name;
        $this->street = $this->data->rp_street;
        $this->postcode = $this->data->rp_postcode;
        $this->countrycode = $this->data->rp_countrycode;
        $this->email = $this->data->rp_email;
        $this->phone = $this->data->rp_phone;
        $this->city=$this->data->rp_city;
    }
    function verify()
    {
        if ( trim($this->name) == "") return;
        /* -- email must be valide */
        if (!preg_match( '/^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})$/',$this->email))
        {
            throw new \Exception(_('Email incorrect'),20);
        }
        if ($this->countrycode != 'BE') {
            throw new \Exception(_('Code Pays incorrect'),21);
        }
        
        if ( ! preg_match('/^[0-9]+/',$this->phone)) {
            throw new \Exception(_('Numéro de téléphone incorrect'),22);
        }
        if (trim($this->street) == "" ) {
            throw new \Exception(_('Obligatoire'),31 );
        }
        if (trim($this->postcode) == "" ) {
            throw new \Exception(_('Obligatoire'),32 );
        }
        if (trim($this->city) == "" ) {
            throw new \Exception(_('Obligatoire'),33 );
        }
    }

}
